@@ -429,10 +429,8 @@ lws_mqtt_create_sub(struct _lws_mqtt_related *mqtt, const char *topic)
 				 __func__);
 			return NULL;
 		}
-		if (flag == LMVTR_VALID_WILDCARD)
-			mysub->wildcard = 1;
-		else if (flag == LMVTR_VALID_SHADOW)
-			mysub->shadow = 1;
+		mysub->wildcard = (flag == LMVTR_VALID_WILDCARD);
+		mysub->shadow = (flag == LMVTR_VALID_SHADOW);
 		break;
 
 	default:
@@ -2038,17 +2036,12 @@ lws_mqtt_client_send_publish(struct lws *wsi, lws_mqtt_publish_param_t *pub,
 			return 1;
 		}
 	}
-	/*
-	 * A non-empty Payload is expected and a chunk
-	 * is present
-	 */
-	if (pub->payload_len && len) {
-		p = lws_mqtt_str_next(&mqtt_vh_payload, NULL);
-		memcpy(p, buf, len);
-		if (lws_mqtt_str_advance(&mqtt_vh_payload, (int)len))
-			return 1;
-		p = lws_mqtt_str_next(&mqtt_vh_payload, NULL);
-	}
+
+	p = lws_mqtt_str_next(&mqtt_vh_payload, NULL);
+	memcpy(p, buf, len);
+	if (lws_mqtt_str_advance(&mqtt_vh_payload, (int)len))
+		return 1;
+	p = lws_mqtt_str_next(&mqtt_vh_payload, NULL);
 
 	if (!is_complete)
 		nwsi->mqtt->inside_payload = wsi->mqtt->inside_payload = 1;
@@ -2272,6 +2265,9 @@ lws_mqtt_client_send_subcribe(struct lws *wsi, lws_mqtt_subscribe_param_t *sub)
 		return 1;
 	}
 
+	if (wsi->mqtt->inside_resume_session)
+		return 0;
+
 	if (lws_write(nwsi, start, lws_ptr_diff_size_t(p, start), LWS_WRITE_BINARY) !=
 					lws_ptr_diff(p, start))
 		return 1;
@@ -2364,8 +2360,10 @@ lws_mqtt_client_send_unsubcribe(struct lws *wsi,
 
 		wsi->mqtt->sub_size = (uint16_t)rem_len;
 
+#if defined(_DEBUG)
 		lwsl_debug("%s: Number of topics = %d, Remaining len = %d\n",
 			   __func__, (int)tops, (int)rem_len);
+#endif
 
 		p += lws_mqtt_vbi_encode(rem_len, p);
 
