@@ -66,8 +66,13 @@ secstream_h2(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 		return n;
 
 	case LWS_CALLBACK_CLOSED_CLIENT_HTTP:
-		if (lws_get_network_wsi(wsi) == wsi)
+		/*
+		 * Only allow the wsi that the handle believes is representing
+		 * him to report closure up to h1
+		 */
+		if (!h || h->wsi != wsi)
 			return 0;
+
 		break;
 
 	case LWS_CALLBACK_COMPLETED_CLIENT_HTTP:
@@ -79,8 +84,7 @@ secstream_h2(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 		r = 0;
 		if (h->hanging_som)
 			r = h->info.rx(ss_to_userobj(h), NULL, 0, LWSSS_FLAG_EOM);
-		/* decouple the fates of the wsi and the ss */
-		h->wsi = NULL;
+
 		h->txn_ok = 1;
 		lws_cancel_service(lws_get_context(wsi)); /* abort poll wait */
 		if (h->hanging_som && r == LWSSSSRET_DESTROY_ME)
@@ -116,8 +120,7 @@ secstream_h2(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 const struct lws_protocols protocol_secstream_h2 = {
 	"lws-secstream-h2",
 	secstream_h2,
-	0,
-	0,
+	0, 0, 0, NULL, 0
 };
 
 /*
@@ -155,6 +158,9 @@ secstream_connect_munge_h2(lws_ss_handle_t *h, char *buf, size_t len,
 	if (h->policy->flags & LWSSSPOLF_HTTP_X_WWW_FORM_URLENCODED)
 		i->ssl_connection |= LCCSCF_HTTP_X_WWW_FORM_URLENCODED;
 
+	if (h->policy->flags & LWSSSPOLF_HTTP_CACHE_COOKIES)
+		i->ssl_connection |= LCCSCF_CACHE_COOKIES;
+
 	i->ssl_connection |= LCCSCF_PIPELINE;
 
 	i->alpn = "h2";
