@@ -46,15 +46,179 @@ secstream_mqtt_cleanup(lws_ss_handle_t *h)
 	}
 }
 
+static int
+secstream_mqtt_subscribe(struct lws *wsi)
+{
+	size_t used_in, used_out, topic_limit;
+	lws_strexp_t exp;
+	char* expbuf;
+	lws_ss_handle_t *h = (lws_ss_handle_t *)lws_get_opaque_user_data(wsi);
+
+	if (!h || !h->policy)
+		return -1;
+
+	if (h->policy->u.mqtt.aws_iot)
+		topic_limit = LWS_MQTT_MAX_AWSIOT_TOPICLEN;
+	else
+		topic_limit = LWS_MQTT_MAX_TOPICLEN;
+
+	if (!h->policy->u.mqtt.subscribe || wsi->mqtt->done_subscribe)
+		return 0;
+
+	lws_strexp_init(&exp, (void*)h, lws_ss_exp_cb_metadata, NULL,
+			topic_limit);
+	/*
+	 * Expand with no output first to calculate the size of
+	 * expanded string then, allocate new buffer and expand
+	 * again with the buffer
+	 */
+	if (lws_strexp_expand(&exp, h->policy->u.mqtt.subscribe,
+			      strlen(h->policy->u.mqtt.subscribe), &used_in,
+			      &used_out) != LSTRX_DONE) {
+		lwsl_err(
+			"%s, failed to expand MQTT subscribe"
+			" topic with no output\n",
+			__func__);
+		return 1;
+	}
+
+	expbuf = lws_malloc(used_out + 1, __func__);
+	if (!expbuf) {
+		lwsl_err(
+			 "%s, failed to allocate MQTT subscribe"
+			 "topic",
+			 __func__);
+		return 1;
+	}
+
+	lws_strexp_init(&exp, (void*)h, lws_ss_exp_cb_metadata, expbuf,
+			used_out + 1);
+
+	if (lws_strexp_expand(&exp, h->policy->u.mqtt.subscribe,
+			      strlen(h->policy->u.mqtt.subscribe), &used_in,
+			      &used_out) != LSTRX_DONE) {
+		lwsl_err("%s, failed to expand MQTT subscribe topic\n",
+			 __func__);
+		lws_free(expbuf);
+		return 1;
+	}
+	lwsl_notice("%s, expbuf - %s\n", __func__, expbuf);
+	h->u.mqtt.sub_top.name = expbuf;
+
+	/*
+	 * The policy says to subscribe to something, and we
+	 * haven't done it yet.  Do it using the pre-prepared
+	 * string-substituted version of the policy string.
+	 */
+
+	lwsl_notice("%s: subscribing %s\n", __func__,
+		    h->u.mqtt.sub_top.name);
+
+	h->u.mqtt.sub_top.qos = h->policy->u.mqtt.qos;
+	memset(&h->u.mqtt.sub_info, 0, sizeof(h->u.mqtt.sub_info));
+	h->u.mqtt.sub_info.num_topics = 1;
+	h->u.mqtt.sub_info.topic = &h->u.mqtt.sub_top;
+	h->u.mqtt.sub_info.topic =
+			    lws_malloc(sizeof(lws_mqtt_topic_elem_t), __func__);
+	h->u.mqtt.sub_info.topic[0].name = lws_strdup(expbuf);
+	h->u.mqtt.sub_info.topic[0].qos = h->policy->u.mqtt.qos;
+
+	if (lws_mqtt_client_send_subcribe(wsi, &h->u.mqtt.sub_info)) {
+		lwsl_notice("%s: unable to subscribe", __func__);
+		lws_free(expbuf);
+		h->u.mqtt.sub_top.name = NULL;
+		return -1;
+	}
+	lws_free(expbuf);
+	h->u.mqtt.sub_top.name = NULL;
+
+	/* Expect a SUBACK */
+	if (lws_change_pollfd(wsi, 0, LWS_POLLIN)) {
+		lwsl_err("%s: Unable to set LWS_POLLIN\n", __func__);
+		return -1;
+	}
+	return 0;
+}
+
+static int
+secstream_mqtt_publish(struct lws *wsi, uint8_t *buf, size_t buflen,
+			const char* topic,
+			lws_mqtt_qos_levels_t qos,  int f)
+{
+	lws_ss_handle_t *h = (lws_ss_handle_t *)lws_get_opaque_user_data(wsi);
+	size_t used_in, used_out, topic_limit;
+	lws_strexp_t exp;
+	char *expbuf;
+	lws_mqtt_publish_param_t mqpp;
+
+	if (h->policy->u.mqtt.aws_iot)
+		topic_limit = LWS_MQTT_MAX_AWSIOT_TOPICLEN;
+	else
+		topic_limit = LWS_MQTT_MAX_TOPICLEN;
+
+	memset(&mqpp, 0, sizeof(mqpp));
+
+	lws_strexp_init(&exp, h, lws_ss_exp_cb_metadata, NULL,
+			topic_limit);
+
+	if (lws_strexp_expand(&exp, topic, strlen(topic), &used_in,
+			      &used_out) != LSTRX_DONE) {
+		lwsl_err("%s, failed to expand MQTT publish"
+			 " topic with no output\n", __func__);
+		return 1;
+	}
+	expbuf = lws_malloc(used_out + 1, __func__);
+	if (!expbuf) {
+		lwsl_err("%s, failed to allocate MQTT publish topic",
+			  __func__);
+		return 1;
+	}
+
+	lws_strexp_init(&exp, (void *)h, lws_ss_exp_cb_metadata, expbuf,
+			used_out + 1);
+
+	if (lws_strexp_expand(&exp, topic, strlen(topic), &used_in,
+			      &used_out) != LSTRX_DONE) {
+		lws_free(expbuf);
+		return 1;
+	}
+	lwsl_notice("%s, expbuf - %s\n", __func__, expbuf);
+	mqpp.topic = (char *)expbuf;
+
+	mqpp.topic_len = (uint16_t)strlen(mqpp.topic);
+	mqpp.packet_id = (uint16_t)(h->txord - 1);
+	mqpp.payload = buf;
+	if (h->writeable_len)
+		mqpp.payload_len = (uint32_t)h->writeable_len;
+	else
+		mqpp.payload_len = (uint32_t)buflen;
+
+	lwsl_notice("%s: payload len %d\n", __func__,
+		    (int)mqpp.payload_len);
+
+	mqpp.qos = h->policy->u.mqtt.qos;
+
+	if (lws_mqtt_client_send_publish(wsi, &mqpp,
+					 (const char *)buf,
+					 (uint32_t)buflen,
+					 f & LWSSS_FLAG_EOM)) {
+		lwsl_notice("%s: failed to publish\n", __func__);
+		lws_free(expbuf);
+		return -1;
+	}
+	lws_free(expbuf);
+	return 0;
+}
+
 static int
 secstream_mqtt(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 	     void *in, size_t len)
 {
 	lws_ss_handle_t *h = (lws_ss_handle_t *)lws_get_opaque_user_data(wsi);
-	lws_mqtt_publish_param_t mqpp, *pmqpp;
+	lws_mqtt_publish_param_t *pmqpp;
 	uint8_t buf[LWS_PRE + 1400];
 	lws_ss_state_return_t r;
-	size_t buflen;
+	size_t buflen = sizeof(buf) - LWS_PRE;
 	int f = 0;
 
 	switch (reason) {
@@ -120,13 +284,32 @@ secstream_mqtt(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 		h->wsi = wsi;
 		h->retry = 0;
 		h->seqstate = SSSEQ_CONNECTED;
-		/*
-		 * If a subscribe is pending on the stream, then make
-		 * sure the SUBSCRIBE is done before signaling the
-		 * user application.
-		 */
-		if (h->policy->u.mqtt.subscribe &&
-		    !wsi->mqtt->done_subscribe) {
+
+		if (!h->policy->u.mqtt.subscribe ||
+		    !h->policy->u.mqtt.subscribe[0]) {
+			/*
+			 * If subscribe is empty in the policy, then,
+			 * skip sending SUBSCRIBE and signal the user
+			 * application.
+			 */
+			wsi->mqtt->done_subscribe = 1;
+		} else if (!h->policy->u.mqtt.clean_start &&
+			   wsi->mqtt->session_resumed) {
+			wsi->mqtt->inside_resume_session = 1;
+			/*
+			 * If the previous session is resumed and Server has
+			 * stored session, then, do not subscribe.
+			 */
+			if (!secstream_mqtt_subscribe(wsi))
+				wsi->mqtt->done_subscribe = 1;
+			wsi->mqtt->inside_resume_session = 0;
+		} else if (h->policy->u.mqtt.subscribe &&
+			   !wsi->mqtt->done_subscribe) {
+			/*
+			 * If a subscribe is pending on the stream, then make
+			 * sure the SUBSCRIBE is done before signaling the
+			 * user application.
+			 */
 			lws_callback_on_writable(wsi);
 			break;
 		}
@@ -175,8 +358,7 @@ secstream_mqtt(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 			lws_sul_cancel(&h->sul);
 			r = lws_ss_event_helper(h, LWSSSCS_CONNECTED);
 			if (r != LWSSSSRET_OK)
-				return _lws_ss_handle_state_ret_CAN_DESTROY_HANDLE(r,
-									      wsi, &h);
+				return _lws_ss_handle_state_ret_CAN_DESTROY_HANDLE(r, wsi, &h);
 		}
 		wsi->mqtt->done_subscribe = 1;
 		lws_callback_on_writable(wsi);
@@ -184,6 +366,14 @@ secstream_mqtt(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 
 	case LWS_CALLBACK_MQTT_ACK:
 		lws_sul_cancel(&h->sul_timeout);
+		if (wsi->mqtt->inside_birth) {
+			/*
+			 * Skip LWSSSCS_QOS_ACK_REMOTE for birth topic.
+			 */
+			wsi->mqtt->inside_birth = 0;
+			wsi->mqtt->done_birth = 1;
+			break;
+		}
 		r = lws_ss_event_helper(h, LWSSSCS_QOS_ACK_REMOTE);
 		if (r != LWSSSSRET_OK)
 			return _lws_ss_handle_state_ret_CAN_DESTROY_HANDLE(r, wsi, &h);
@@ -191,10 +381,6 @@ secstream_mqtt(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 
 	case LWS_CALLBACK_MQTT_CLIENT_WRITEABLE:
 	{
-		size_t used_in, used_out, topic_limit;
-		lws_strexp_t exp;
-		char *expbuf;
-
 		if (!h || !h->info.tx)
 			return 0;
 		lwsl_notice("%s: %s: WRITEABLE\n", __func__, lws_ss_tag(h));
@@ -203,88 +389,29 @@ secstream_mqtt(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 			lwsl_warn("%s: seqstate %d\n", __func__, h->seqstate);
 			break;
 		}
-		if (h->policy->u.mqtt.aws_iot)
-			topic_limit = LWS_MQTT_MAX_AWSIOT_TOPICLEN;
-		else
-			topic_limit = LWS_MQTT_MAX_TOPICLEN;
 
-		if (h->policy->u.mqtt.subscribe &&
-		    !wsi->mqtt->done_subscribe) {
-			lws_strexp_init(&exp, (void *)h, lws_ss_exp_cb_metadata,
-					NULL, topic_limit);
-			/*
-			 * Expand with no output first to calculate the size of
-			 * expanded string then, allocate new buffer and expand
-			 * again with the buffer
-			 */
-			if (lws_strexp_expand(&exp, h->policy->u.mqtt.subscribe,
-					      strlen(h->policy->u.mqtt.subscribe),
-					      &used_in, &used_out) != LSTRX_DONE) {
-				lwsl_err("%s, failed to expand MQTT subscribe"
-					 " topic with no output\n", __func__);
-				return 1;
+		if (!wsi->mqtt->done_subscribe && h->policy->u.mqtt.subscribe)
+			return secstream_mqtt_subscribe(wsi);
+
+		if (!wsi->mqtt->done_birth && h->policy->u.mqtt.birth_topic) {
+			lws_strexp_t exp;
+			size_t used_in, used_out = 0;
+			if (h->policy->u.mqtt.birth_message) {
+				lws_strexp_init(&exp, h, lws_ss_exp_cb_metadata,
+						(char *)(buf + LWS_PRE), buflen);
+				if (lws_strexp_expand(&exp, h->policy->u.mqtt.birth_message,
+						      strlen(h->policy->u.mqtt.birth_message),
+						      &used_in, &used_out) != LSTRX_DONE) {
+					return 1;
+				}
 			}
-
-			expbuf = lws_malloc(used_out + 1, __func__);
-			if (!expbuf) {
-				lwsl_err("%s, failed to allocate MQTT subscribe"
-				         "topic", __func__);
-				return 1;
-			}
-
-			lws_strexp_init(&exp, (void *)h, lws_ss_exp_cb_metadata,
-					expbuf, used_out + 1);
-
-			if (lws_strexp_expand(&exp, h->policy->u.mqtt.subscribe,
-					      strlen(h->policy->u.mqtt.subscribe),
-					      &used_in, &used_out) != LSTRX_DONE) {
-				lwsl_err("%s, failed to expand MQTT subscribe topic\n",
-					 __func__);
-				lws_free(expbuf);
-				return 1;
-			}
-			lwsl_notice("%s, expbuf - %s\n", __func__, expbuf);
-			h->u.mqtt.sub_top.name = expbuf;
-
-			/*
-			 * The policy says to subscribe to something, and we
-			 * haven't done it yet.  Do it using the pre-prepared
-			 * string-substituted version of the policy string.
-			 */
-
-			lwsl_notice("%s: subscribing %s\n", __func__,
-				                h->u.mqtt.sub_top.name);
-
-			h->u.mqtt.sub_top.qos = h->policy->u.mqtt.qos;
-			memset(&h->u.mqtt.sub_info, 0, sizeof(h->u.mqtt.sub_info));
-			h->u.mqtt.sub_info.num_topics = 1;
-			h->u.mqtt.sub_info.topic = &h->u.mqtt.sub_top;
-			h->u.mqtt.sub_info.topic = lws_malloc(sizeof(lws_mqtt_topic_elem_t),
-							      __func__);
-			h->u.mqtt.sub_info.topic[0].name = lws_strdup(expbuf);
-			h->u.mqtt.sub_info.topic[0].qos = h->policy->u.mqtt.qos;
-
-			if (lws_mqtt_client_send_subcribe(wsi, &h->u.mqtt.sub_info)) {
-				lwsl_notice("%s: unable to subscribe", __func__);
-				lws_free(expbuf);
-				h->u.mqtt.sub_top.name = NULL;
-				return -1;
-			}
-			lws_free(expbuf);
-			h->u.mqtt.sub_top.name = NULL;
-			/* Expect a SUBACK */
-			if (lws_change_pollfd(wsi, 0, LWS_POLLIN)) {
-				lwsl_err("%s: Unable to set LWS_POLLIN\n", __func__);
-				return -1;
-			}
-
-			return 0;
+			wsi->mqtt->inside_birth = 1;
+			return secstream_mqtt_publish(wsi, buf + LWS_PRE,
+					used_out, h->policy->u.mqtt.birth_topic,
+					h->policy->u.mqtt.birth_qos, LWSSS_FLAG_EOM);
 		}
-
-
-		buflen = sizeof(buf) - LWS_PRE;
 		r = h->info.tx(ss_to_userobj(h),  h->txord++,  buf + LWS_PRE,
-				  &buflen, &f);
+			       &buflen, &f);
 		if (r == LWSSSSRET_TX_DONT_SEND)
 			return 0;
 
@@ -307,63 +434,9 @@ secstream_mqtt(struct lws *wsi, enum lws_callback_reasons reason, void *user,
 		if (r < 0)
 			return _lws_ss_handle_state_ret_CAN_DESTROY_HANDLE(r, wsi, &h);
 
-		memset(&mqpp, 0, sizeof(mqpp));
-		/* this is the string-substituted h->policy->u.mqtt.topic */
-		mqpp.topic = (char *)h->u.mqtt.topic_qos.name;
-		lws_strexp_init(&exp, h, lws_ss_exp_cb_metadata, NULL,
-				topic_limit);
-
-		if (lws_strexp_expand(&exp, h->policy->u.mqtt.topic,
-				      strlen(h->policy->u.mqtt.topic),
-				      &used_in, &used_out) != LSTRX_DONE) {
-			lwsl_err("%s, failed to expand MQTT publish"
-				 " topic with no output\n", __func__);
-			return 1;
-		}
-		expbuf = lws_malloc(used_out + 1, __func__);
-		if (!expbuf) {
-			lwsl_err("%s, failed to allocate MQTT publish topic",
-				 __func__);
-			return 1;
-		}
-
-		lws_strexp_init(&exp, (void *)h, lws_ss_exp_cb_metadata, expbuf,
-				used_out + 1);
-
-                if (lws_strexp_expand(&exp, h->policy->u.mqtt.topic,
-				      strlen(h->policy->u.mqtt.topic), &used_in,
-				      &used_out) != LSTRX_DONE) {
-			lws_free(expbuf);
-			return 1;
-		}
-		lwsl_notice("%s, expbuf - %s\n", __func__, expbuf);
-		mqpp.topic = (char *)expbuf;
-
-		mqpp.topic_len = (uint16_t)strlen(mqpp.topic);
-		mqpp.packet_id = (uint16_t)(h->txord - 1);
-		mqpp.payload = buf + LWS_PRE;
-		if (h->writeable_len)
-			mqpp.payload_len = (uint32_t)h->writeable_len;
-		else
-			mqpp.payload_len = (uint32_t)buflen;
-
-		lwsl_notice("%s: payload len %d\n", __func__,
-				(int)mqpp.payload_len);
-
-		mqpp.qos = h->policy->u.mqtt.qos;
-
-		if (lws_mqtt_client_send_publish(wsi, &mqpp,
-						 (const char *)buf + LWS_PRE,
-						 (uint32_t)buflen,
-						 f & LWSSS_FLAG_EOM)) {
-			lwsl_notice("%s: failed to publish\n", __func__);
-			lws_free(expbuf);
-
-			return -1;
-		}
-		lws_free(expbuf);
-
-		return 0;
+		return secstream_mqtt_publish(wsi, buf + LWS_PRE, buflen,
+					      h->policy->u.mqtt.topic,
+					      h->policy->u.mqtt.qos, f);
 	}
 
 	case LWS_CALLBACK_MQTT_UNSUBSCRIBED:
@@ -409,7 +482,9 @@ enum {
 	SSCMM_STRSUB_WILL_TOPIC,
 	SSCMM_STRSUB_WILL_MESSAGE,
 	SSCMM_STRSUB_SUBSCRIBE,
-	SSCMM_STRSUB_TOPIC
+	SSCMM_STRSUB_TOPIC,
+	SSCMM_STRSUB_BIRTH_TOPIC,
+	SSCMM_STRSUB_BIRTH_MESSAGE
 };
 
 static int
@@ -417,16 +492,18 @@ secstream_connect_munge_mqtt(lws_ss_handle_t *h, char *buf, size_t len,
 			     struct lws_client_connect_info *i,
 			     union lws_ss_contemp *ct)
 {
-	const char *sources[4] = {
+	const char *sources[6] = {
 		/* we're going to string-substitute these before use */
 		h->policy->u.mqtt.will_topic,
 		h->policy->u.mqtt.will_message,
 		h->policy->u.mqtt.subscribe,
-		h->policy->u.mqtt.topic
+		h->policy->u.mqtt.topic,
+		h->policy->u.mqtt.birth_topic,
+		h->policy->u.mqtt.birth_message
 	};
-	size_t used_in, olen[4] = { 0, 0, 0, 0 }, tot = 0;
+	size_t used_in, olen[6] = { 0, 0, 0, 0, 0, 0 }, tot = 0;
 	lws_strexp_t exp;
-	char *ps[4];
+	char *ps[6];
 	uint8_t *p = NULL;
 	int n = -1;
 	size_t blen;
@@ -444,6 +521,8 @@ secstream_connect_munge_mqtt(lws_ss_handle_t *h, char *buf, size_t len,
 			return -1;
 		}
 		p = (uint8_t *)lws_zalloc(blen+1, __func__);
+		if (!p)
+			return -1;
 		n = lws_system_blob_get(b, p, &blen, 0);
 		if (n) {
 			ct->ccp.client_id = NULL;
@@ -463,6 +542,8 @@ secstream_connect_munge_mqtt(lws_ss_handle_t *h, char *buf, size_t len,
 	/* If LWS_SYSBLOB_TYPE_MQTT_USERNAME is set */
 	if (b && (blen = lws_system_blob_get_size(b))) {
 		p = (uint8_t *)lws_zalloc(blen+1, __func__);
+		if (!p)
+			return -1;
 		n = lws_system_blob_get(b, p, &blen, 0);
 		if (n) {
 			ct->ccp.username = NULL;
@@ -479,6 +560,8 @@ secstream_connect_munge_mqtt(lws_ss_handle_t *h, char *buf, size_t len,
 	/* If LWS_SYSBLOB_TYPE_MQTT_PASSWORD is set */
 	if (b && (blen = lws_system_blob_get_size(b))) {
 		p = (uint8_t *)lws_zalloc(blen+1, __func__);
+		if (!p)
+			return -1;
 		n = lws_system_blob_get(b, p, &blen, 0);
 		if (n) {
 			ct->ccp.password = NULL;
@@ -493,6 +576,8 @@ secstream_connect_munge_mqtt(lws_ss_handle_t *h, char *buf, size_t len,
 	ct->ccp.clean_start		= (h->policy->u.mqtt.clean_start & 1u);
 	ct->ccp.will_param.qos		= h->policy->u.mqtt.will_qos;
 	ct->ccp.will_param.retain	= h->policy->u.mqtt.will_retain;
+	ct->ccp.birth_param.qos		= h->policy->u.mqtt.birth_qos;
+	ct->ccp.birth_param.retain	= h->policy->u.mqtt.birth_retain;
 	ct->ccp.aws_iot			= h->policy->u.mqtt.aws_iot;
 	h->u.mqtt.topic_qos.qos		= h->policy->u.mqtt.qos;
 
@@ -513,6 +598,9 @@ secstream_connect_munge_mqtt(lws_ss_handle_t *h, char *buf, size_t len,
 	 */
 
 	for (n = 0; n < (int)LWS_ARRAY_SIZE(sources); n++) {
+		if (!sources[n])
+			continue;
+
 		lws_strexp_init(&exp, (void *)h, lws_ss_exp_cb_metadata,
 				NULL, (size_t)-1);
 		if (lws_strexp_expand(&exp, sources[n], strlen(sources[n]),
@@ -564,6 +652,8 @@ secstream_connect_munge_mqtt(lws_ss_handle_t *h, char *buf, size_t len,
 	h->u.mqtt.subscribe_to		= ps[SSCMM_STRSUB_SUBSCRIBE];
 	h->u.mqtt.subscribe_to_len	= olen[SSCMM_STRSUB_SUBSCRIBE];
 	h->u.mqtt.topic_qos.name	= ps[SSCMM_STRSUB_TOPIC];
+	ct->ccp.birth_param.topic	= ps[SSCMM_STRSUB_BIRTH_TOPIC];
+	ct->ccp.birth_param.message	= ps[SSCMM_STRSUB_BIRTH_MESSAGE];
 
 	i->method = "MQTT";
 	i->mqtt_cp = &ct->ccp;
