@@ -863,11 +863,12 @@ _lws_ss_client_connect(lws_ss_handle_t *h, int is_retry, void *conn_if_sspc_onw)
 
 #if defined(LWS_WITH_SYS_METRICS)
 	/* possibly already hanging connect retry... */
-	if (!h->cal_txn.mt) {
+	if (!h->cal_txn.mt)
 		lws_metrics_caliper_bind(h->cal_txn, h->context->mth_ss_conn);
-	}
 
-	lws_metrics_tag_add(&h->cal_txn.mtags_owner, "ss", h->policy->streamtype);
+	if (h->policy->streamtype)
+		lws_metrics_tag_add(&h->cal_txn.mtags_owner, "ss",
+				    h->policy->streamtype);
 #endif
 
 	h->txn_ok = 0;
@@ -1282,6 +1283,14 @@ extant:
 		lwsl_err("%s: unable to get vhost / trust store\n", __func__);
 		goto fail_creation;
 	}
+#else
+#if defined(LWS_WITH_SECURE_STREAMS_CPP)
+        if (!ssi->streamtype &&
+	    !lws_ss_policy_ref_trust_store(context, h->policy, 1 /* do the ref */)) {
+		lwsl_err("%s: unable to get vhost / trust store\n", __func__);
+		goto fail_creation;
+	}
+#endif
 #endif
 
 	r = lws_ss_event_helper(h, LWSSSCS_CREATING);
@@ -1486,6 +1495,11 @@ lws_ss_destroy(lws_ss_handle_t **ppss)
 
 	if (h->policy)
 		lws_ss_policy_unref_trust_store(h->context, h->policy);
+#else
+#if defined(LWS_WITH_SECURE_STREAMS_CPP)
+	if (!h->info.streamtype || !*(h->info.streamtype))
+		lws_ss_policy_unref_trust_store(h->context, h->policy);
+#endif
 #endif
 
 #if defined(LWS_WITH_SERVER)
@@ -1731,10 +1745,8 @@ lws_ss_to_cb(lws_sorted_usec_list_t *sul)
 	if (r != LWSSSSRET_DISCONNECT_ME && r != LWSSSSRET_DESTROY_ME)
 		return;
 
-	if (!h->wsi)
-		return;
-
-	lws_set_timeout(h->wsi, 1, LWS_TO_KILL_ASYNC);
+	if (h->wsi)
+		lws_set_timeout(h->wsi, 1, LWS_TO_KILL_ASYNC);
 
 	_lws_ss_handle_state_ret_CAN_DESTROY_HANDLE(r, h->wsi, &h);
 }
