@@ -61,7 +61,7 @@ OpenSSL_verify_callback(int preverify_ok, X509_STORE_CTX *x509_ctx)
 
 	n = wsi->a.vhost->protocols[0].callback(wsi,
 			LWS_CALLBACK_OPENSSL_PERFORM_CLIENT_CERT_VERIFICATION,
-					   x509_ctx, ssl, preverify_ok);
+					   x509_ctx, ssl, (unsigned int)preverify_ok);
 
 	/* convert return code from 0 = OK to 1 = OK */
 	return !n;
@@ -155,7 +155,9 @@ lws_tls_server_certs_load(struct lws_vhost *vhost, struct lws *wsi,
 			  const char *mem_cert, size_t mem_cert_len,
 			  const char *mem_privkey, size_t mem_privkey_len)
 {
-#if !defined(OPENSSL_NO_EC)
+#if !defined(OPENSSL_NO_EC) && defined(LWS_HAVE_EC_KEY_new_by_curve_name) && \
+    ((OPENSSL_VERSION_NUMBER < 0x30000000l) || \
+     defined(LWS_SUPPRESS_DEPRECATED_API_WARNINGS))
 	const char *ecdh_curve = "prime256v1";
 #if !defined(LWS_WITH_BORINGSSL) && defined(LWS_HAVE_SSL_EXTRA_CHAIN_CERTS)
 	STACK_OF(X509) *extra_certs = NULL;
@@ -172,7 +174,7 @@ lws_tls_server_certs_load(struct lws_vhost *vhost, struct lws *wsi,
 #if OPENSSL_VERSION_NUMBER >= 0x10100000L
 	int ret;
 #endif
-	int n = lws_tls_generic_cert_checks(vhost, cert, private_key), m;
+	int n = (int)lws_tls_generic_cert_checks(vhost, cert, private_key), m;
 
 	if (!cert && !private_key)
 		n = LWS_TLS_EXTANT_ALTERNATIVE;
@@ -210,10 +212,18 @@ lws_tls_server_certs_load(struct lws_vhost *vhost, struct lws *wsi,
 		/* set the local certificate from CertFile */
 		m = SSL_CTX_use_certificate_chain_file(vhost->tls.ssl_ctx, cert);
 		if (m != 1) {
+			const char *s;
 			error = ERR_get_error();
+
+			s = ERR_error_string(
+#if defined(LWS_WITH_BORINGSSL)
+				(uint32_t)
+#endif
+					error,
+				       (char *)vhost->context->pt[0].serv_buf);
+
 			lwsl_err("problem getting cert '%s' %lu: %s\n",
-				 cert, error, ERR_error_string(error,
-				       (char *)vhost->context->pt[0].serv_buf));
+				 cert, error, s);
 
 			return 1;
 		}
@@ -222,11 +232,16 @@ lws_tls_server_certs_load(struct lws_vhost *vhost, struct lws *wsi,
 			/* set the private key from KeyFile */
 			if (SSL_CTX_use_PrivateKey_file(vhost->tls.ssl_ctx, private_key,
 							SSL_FILETYPE_PEM) != 1) {
+				const char *s;
 				error = ERR_get_error();
+				s = ERR_error_string(
+	#if defined(LWS_WITH_BORINGSSL)
+					(uint32_t)
+	#endif
+						error,
+					       (char *)vhost->context->pt[0].serv_buf);
 				lwsl_err("ssl problem getting key '%s' %lu: %s\n",
-					 private_key, error,
-					 ERR_error_string(error,
-					      (char *)vhost->context->pt[0].serv_buf));
+					 private_key, error, s);
 				return 1;
 			}
 		} else {
@@ -252,7 +267,13 @@ lws_tls_server_certs_load(struct lws_vhost *vhost, struct lws *wsi,
 	}
 
 #if !defined(USE_WOLFSSL)
-	ret = SSL_CTX_use_certificate_ASN1(vhost->tls.ssl_ctx, (int)flen, p);
+	ret = SSL_CTX_use_certificate_ASN1(vhost->tls.ssl_ctx,
+#if defined(LWS_WITH_BORINGSSL)
+				(size_t)
+#else
+				(int)
+#endif
+				flen, p);
 #else
 	ret = wolfSSL_CTX_use_certificate_buffer(vhost->tls.ssl_ctx,
 						 (uint8_t *)p, (int)flen,
@@ -275,11 +296,21 @@ lws_tls_server_certs_load(struct lws_vhost *vhost, struct lws *wsi,
 
 #if !defined(USE_WOLFSSL)
 	ret = SSL_CTX_use_PrivateKey_ASN1(EVP_PKEY_RSA, vhost->tls.ssl_ctx, p,
-					  (long)(long long)flen);
+#if defined(LWS_WITH_BORINGSSL)
+			(size_t)
+#else
+					  (long)(long long)
+#endif
+					  flen);
 	if (ret != 1) {
 		ret = SSL_CTX_use_PrivateKey_ASN1(EVP_PKEY_EC,
 						  vhost->tls.ssl_ctx, p,
-						  (long)(long long)flen);
+#if defined(LWS_WITH_BORINGSSL)
+			(size_t)
+#else
+					  (long)(long long)
+#endif
+						  flen);
 	}
 #else
 	ret = wolfSSL_CTX_use_PrivateKey_buffer(vhost->tls.ssl_ctx, p, flen,
@@ -337,7 +368,7 @@ lws_tls_server_certs_load(struct lws_vhost *vhost, struct lws *wsi,
 						(long)(long long)flen) != 1) {
 #else
 		if (wolfSSL_CTX_use_PrivateKey_buffer(vhost->tls.ssl_ctx, p,
-					    flen, WOLFSSL_FILETYPE_ASN1) != 1) {
+					    (long)flen, WOLFSSL_FILETYPE_ASN1) != 1) {
 #endif
 			lwsl_notice("unable to use memory privkey\n");
 
@@ -390,7 +421,9 @@ check_key:
 	}
 
 
-#if !defined(OPENSSL_NO_EC)
+#if !defined(OPENSSL_NO_EC) && defined(LWS_HAVE_EC_KEY_new_by_curve_name) && \
+    ((OPENSSL_VERSION_NUMBER < 0x30000000l) || \
+     defined(LWS_SUPPRESS_DEPRECATED_API_WARNINGS))
 	if (vhost->tls.ecdh_curve[0])
 		ecdh_curve = vhost->tls.ecdh_curve;
 
@@ -432,7 +465,8 @@ check_key:
 	}
 #else
 	return 0;
-#endif
+#endif /* !boringssl */
+
 	/* Get the public key from certificate */
 	pkey = X509_get_pubkey(x);
 	if (!pkey) {
@@ -457,13 +491,14 @@ check_key:
 	SSL_CTX_set_tmp_ecdh(vhost->tls.ssl_ctx, EC_key);
 
 	EC_KEY_free(EC_key);
-#else
-	lwsl_notice(" OpenSSL doesn't support ECDH\n");
-#endif
+
 #if !defined(OPENSSL_NO_EC) && !defined(LWS_WITH_BORINGSSL)
 post_ecdh:
 #endif
 	vhost->tls.skipped_certs = 0;
+#else
+	lwsl_notice(" OpenSSL doesn't support ECDH\n");
+#endif
 
 	return 0;
 }
@@ -476,18 +511,32 @@ lws_tls_server_vhost_backend_init(const struct lws_context_creation_info *info,
 	SSL_METHOD *method = (SSL_METHOD *)SSLv23_server_method();
 
 	if (!method) {
+		const char *s;
 		error = ERR_get_error();
+		s = ERR_error_string(
+#if defined(LWS_WITH_BORINGSSL)
+			(uint32_t)
+#endif
+				error,
+			       (char *)vhost->context->pt[0].serv_buf);
+
 		lwsl_err("problem creating ssl method %lu: %s\n",
-				error, ERR_error_string(error,
-				      (char *)vhost->context->pt[0].serv_buf));
+				error, s);
 		return 1;
 	}
 	vhost->tls.ssl_ctx = SSL_CTX_new(method);	/* create context */
 	if (!vhost->tls.ssl_ctx) {
+		const char *s;
+
 		error = ERR_get_error();
+		s = ERR_error_string(
+#if defined(LWS_WITH_BORINGSSL)
+			(uint32_t)
+#endif
+				error,
+			       (char *)vhost->context->pt[0].serv_buf);
 		lwsl_err("problem creating ssl context %lu: %s\n",
-				error, ERR_error_string(error,
-				      (char *)vhost->context->pt[0].serv_buf));
+				error, s);
 		return 1;
 	}
 
@@ -531,12 +580,35 @@ lws_tls_server_vhost_backend_init(const struct lws_context_creation_info *info,
 	}
 
 	if (info->ssl_options_set)
-		SSL_CTX_set_options(vhost->tls.ssl_ctx, info->ssl_options_set);
+		SSL_CTX_set_options(vhost->tls.ssl_ctx,
+#if defined(USE_WOLFSSL)
+				(long)
+#else
+#if defined(LWS_WITH_BORINGSSL)
+				(uint32_t)
+#else
+#if (OPENSSL_VERSION_NUMBER >= 0x10003000l) && !defined(LIBRESSL_VERSION_NUMBER) /* not documented by openssl */
+				    (unsigned long)
+#else
+				    (long)
+#endif
+#endif
+#endif
+				info->ssl_options_set);
 
 /* SSL_clear_options introduced in 0.9.8m */
 #if (OPENSSL_VERSION_NUMBER >= 0x009080df) && !defined(USE_WOLFSSL)
 	if (info->ssl_options_clear)
 		SSL_CTX_clear_options(vhost->tls.ssl_ctx,
+#if defined(LWS_WITH_BORINGSSL)
+				(uint32_t)
+#else
+#if (OPENSSL_VERSION_NUMBER >= 0x10003000l)  && !defined(LIBRESSL_VERSION_NUMBER)/* not documented by openssl */
+				    (unsigned long)
+#else
+				    (long)
+#endif
+#endif
 				      info->ssl_options_clear);
 #endif
 
@@ -610,7 +682,8 @@ lws_tls_server_new_nonblocking(struct lws *wsi, lws_sockfd_type accept_fd)
 int
 lws_tls_server_abort_connection(struct lws *wsi)
 {
-	SSL_shutdown(wsi->tls.ssl);
+	if (wsi->tls.use_ssl)
+		SSL_shutdown(wsi->tls.ssl);
 	SSL_free(wsi->tls.ssl);
 
 	return 0;
@@ -991,7 +1064,7 @@ lws_tls_acme_sni_csr_create(struct lws_context *context, const char *elements[],
 			if (*p == '/')
 				*csr++ = '_';
 			else
-				*csr++ = *p;
+				*csr++ = (uint8_t)*p;
 		p++;
 		csr_len--;
 	}
@@ -1014,7 +1087,7 @@ lws_tls_acme_sni_csr_create(struct lws_context *context, const char *elements[],
 		goto bail3;
 	}
 	bio_len = BIO_get_mem_data(bio, &p);
-	*privkey_pem = malloc(bio_len); /* malloc so user code can own / free */
+	*privkey_pem = malloc((unsigned long)bio_len); /* malloc so user code can own / free */
 	*privkey_len = (size_t)bio_len;
 	if (!*privkey_pem) {
 		lwsl_notice("%s: need %ld for private key\n", __func__,
@@ -1022,7 +1095,7 @@ lws_tls_acme_sni_csr_create(struct lws_context *context, const char *elements[],
 		BIO_free(bio);
 		goto bail3;
 	}
-	memcpy(*privkey_pem, p, (int)(long long)bio_len);
+	memcpy(*privkey_pem, p, (unsigned int)(int)(long long)bio_len);
 	BIO_free(bio);
 
 	ret = lws_ptr_diff(csr, csr_in);
