@@ -32,8 +32,10 @@ lws_ssl_client_connect1(struct lws *wsi, char *errbuf, size_t len)
 	n = lws_tls_client_connect(wsi, errbuf, len);
 	switch (n) {
 	case LWS_SSL_CAPABLE_ERROR:
+		lws_tls_restrict_return_handshake(wsi);
 		return -1;
 	case LWS_SSL_CAPABLE_DONE:
+		lws_tls_restrict_return_handshake(wsi);
 		lws_metrics_caliper_report(wsi->cal_conn, METRES_GO);
 #if defined(LWS_WITH_CONMON)
 	wsi->conmon.ciu_tls = (lws_conmon_interval_us_t)
@@ -63,6 +65,7 @@ lws_ssl_client_connect2(struct lws *wsi, char *errbuf, size_t len)
 
 		switch (n) {
 		case LWS_SSL_CAPABLE_ERROR:
+			lws_tls_restrict_return_handshake(wsi);
 			// lws_snprintf(errbuf, len, "client connect failed");
 			return -1;
 		case LWS_SSL_CAPABLE_DONE:
@@ -74,10 +77,12 @@ lws_ssl_client_connect2(struct lws *wsi, char *errbuf, size_t len)
 			lwsi_set_state(wsi, LRS_WAITING_SSL);
 			/* fallthru */
 		case LWS_SSL_CAPABLE_MORE_SERVICE:
-			return 0;
+			return 0; /* retry */
 		}
 	}
 
+	lws_tls_restrict_return_handshake(wsi);
+
 	if (lws_tls_client_confirm_peer_cert(wsi, errbuf, len)) {
 		lws_metrics_caliper_report(wsi->cal_conn, METRES_NOGO);
 		return -1;
@@ -89,7 +94,7 @@ lws_ssl_client_connect2(struct lws *wsi, char *errbuf, size_t len)
 					(lws_now_usecs() - wsi->conmon_datum);
 #endif
 
-	return 1;
+	return 1; /* connected */
 }
 
 
@@ -176,29 +181,28 @@ int lws_context_init_client_ssl(const struct lws_context_creation_info *info,
 int
 lws_client_create_tls(struct lws *wsi, const char **pcce, int do_c1)
 {
-
 	/* we can retry this... just cook the SSL BIO the first time */
 
 	if (wsi->tls.use_ssl & LCCSCF_USE_SSL) {
 		int n;
 
 		if (!wsi->tls.ssl) {
-			if (lws_ssl_client_bio_create(wsi) < 0) {
-				*pcce = "bio_create failed";
-				return CCTLS_RETURN_ERROR;
-			}
 
 #if defined(LWS_WITH_TLS)
 			if (!wsi->transaction_from_pipeline_queue &&
-			    lws_tls_restrict_borrow(wsi->a.context)) {
+			    lws_tls_restrict_borrow(wsi)) {
 				*pcce = "tls restriction limit";
 				return CCTLS_RETURN_ERROR;
 			}
 #endif
+			if (lws_ssl_client_bio_create(wsi) < 0) {
+				*pcce = "bio_create failed";
+				return CCTLS_RETURN_ERROR;
+			}
 		}
 
 		if (!do_c1)
-			return 0;
+			return CCTLS_RETURN_DONE;
 
 		lws_metrics_caliper_report(wsi->cal_conn, METRES_GO);
 		lws_metrics_caliper_bind(wsi->cal_conn, wsi->a.context->mt_conn_tls);
@@ -211,39 +215,18 @@ lws_client_create_tls(struct lws *wsi, const char **pcce, int do_c1)
 		lwsl_debug("%s: lws_ssl_client_connect1: %d\n", __func__, n);
 		if (!n)
 			return CCTLS_RETURN_RETRY; /* caller should return 0 */
+
 		if (n < 0) {
 			*pcce = (const char *)wsi->a.context->pt[(int)wsi->tsi].serv_buf;
 			lws_metrics_caliper_report(wsi->cal_conn, METRES_NOGO);
 			return CCTLS_RETURN_ERROR;
 		}
 		/* ...connect1 already handled caliper if SSL_accept done */
+
+		lws_tls_server_conn_alpn(wsi);
+
 	} else
 		wsi->tls.ssl = NULL;
 
-#if 0
-#if defined (LWS_WITH_HTTP2)
-	if (wsi->client_h2_alpn) {
-		/*
-		 * We connected to the server and set up tls, and
-		 * negotiated "h2".
-		 *
-		 * So this is it, we are an h2 nwsi client connection
-		 * now, not an h1 client connection.
-		 */
-#if defined(LWS_WITH_TLS)
-		lws_tls_server_conn_alpn(wsi);
-#endif
-
-		/* send the H2 preface to legitimize the connection */
-		if (lws_h2_issue_preface(wsi)) {
-			*pcce = "error sending h2 preface";
-			return CCTLS_RETURN_ERROR;
-		}
-
-		lwsi_set_state(wsi, LRS_H1C_ISSUE_HANDSHAKE2);
-	}
-#endif
-#endif
-
 	return CCTLS_RETURN_DONE; /* OK */
 }
